package org.springblade.cube.aspect;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 数据源切面---执行数据源相关的sql时先判断是否有该数据源
 */
@Aspect
@Component
@Slf4j
public class DsAspect {


    @Autowired
    private DynamicDataSourceSupport dynamicDataSourceSupport;

    private void addDs(ReportDatasource ds) {
        if (!dynamicDataSourceSupport.exists(ds.getPoolName())) {
            //RPC 调用时，只有poolName值
            if (ds.getId() == null && StrUtil.isNotBlank(ds.getPoolName())) {
                //获取数据源service
                String poolName = ds.getPoolName();
                IReportDatasourceService datasourceService = SpringUtil.getBean(IReportDatasourceService.class);
                ds = datasourceService.getCacheByPoolName(poolName);
                if (ds == null) {
                    log.warn("添加数据源|未找到数据源|poolName={}", poolName);
                    return;
                }
            }
            dynamicDataSourceSupport.addDs(ds);
        }
    }

    @Before("execution(* org.springblade.cube.service.DynamicTableService..*(..))")
    public void beforeMethod(JoinPoint joinPoint) {
        Object[] args = joinPoint.getArgs();
        for (Object arg : args) {
            if (arg instanceof ReportDatasource) {
                ReportDatasource ds = (ReportDatasource) arg;
                addDs(ds);
                break;
            }
        }
    }


}
