/**
 * 
 */
package org.springblade.cube.controller;

import java.util.Map;

import org.springblade.bi.client.model.CubeBrowserSqlModel;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.cube.dto.BrowserRequest;
import org.springblade.cube.dto.BrowserResult;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataBrowserService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/cube/browser")
@Api(value = "数据集数据浏览接口", tags = "数据集数据浏览接口")
public class CubeDataBrowserController extends BladeController {

	@Autowired
	private CubeDataService cubeDataService;

	@Autowired
	private IReportDatasourceService reportDatasourceService;

	@Autowired
	private CubeDataBrowserService cubeDataBrowserService;
	
	@Autowired
	private DynamicTableService dynamicTableService;
	
	/**
	 * 
	 * @return
	 */
	@PostMapping("/{cubeId}")
	@ApiOperation(value = "浏览数据接口")
	public R<BrowserResult> browser(@PathVariable Long cubeId, @RequestBody BrowserRequest browserRequest) {
//		//获取分页model和查询参数model
		Query query = browserRequest.getPageQuery();
		
		CubeData cubeData = cubeDataService.getById(cubeId);
		if(cubeData == null) {
			return R.fail("数据集不存在");
		}
//		//检查数据集是否有权限
//		boolean checkCubePermRet = cubeDataPermissionService.checkCubePermForUser(cubeId, cubeData.getCreateUserId(), getUser());
//		if(!checkCubePermRet) {
//			return R.fail("数据集没有访问权限");
//		}
		ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		CubeBrowserSqlModel cubeBrowserSqlModel = cubeDataBrowserService.buildSql(cubeId, browserRequest, getUser());
		//分页查询
		IPage<Map<String, Object>> pageResult = dynamicTableService.selectPage(Condition.getPage(query), reportDatasource, cubeBrowserSqlModel.getSql());
//		List<Map<String, Object>> resultList = pageResult != null ? pageResult.getRecords() : null;
		//列权限，删除掉不显示的列
//		_formatExcludeColumn(resultList, columnList, cubePermParseDto.getExcludeColumnIdList());
		return R.data(new BrowserResult(pageResult, cubeBrowserSqlModel.getColumnDict()), cubeBrowserSqlModel.getSql());
	}
	
	/**
	 * 列权限校验
	 * @param fieldDict
	 * @param resultList
	 * @param columnList
	 * @param excludeColumnIdList
	 */
//	void _formatExcludeColumn(List<Map<String, Object>> resultList, List<CubeTableColumn> columnList, Set<Integer> excludeColumnIdList) {
//		//列权限，删除掉不显示的列
//		if(CollectionUtil.isNotEmpty(excludeColumnIdList) && CollectionUtil.isNotEmpty(resultList)) {
//			Map<Integer, String> fieldIdMap = columnList.stream().collect(Collectors.toMap(CubeTableColumn::getId, CubeTableColumn::getColumnAliasName));
//			List<String> excludeColumnAliasList = excludeColumnIdList.stream().map(it -> fieldIdMap.get(it)).filter(Objects::nonNull).collect(Collectors.toList());
//			if(CollectionUtil.isNotEmpty(excludeColumnAliasList)) {
//				//格式化数据
//				for(Map<String, Object> jsonObj : resultList) {
//					excludeColumnAliasList.stream().forEach(item -> jsonObj.remove(item));
//				}
//			}
//		}
//	}
}
