package org.springblade.visual.controller;

import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.constant.CacheConstants;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.redis.cache.CacheKey;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.model.TenantModel;
import org.springblade.visual.dto.ComponentLineChartDto;
import org.springblade.visual.dto.ComponentStatisticsParams;
import org.springblade.visual.dto.ComponentTopDto;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.service.ComponentStatisticsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.time.Duration;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Api(tags = "组件统计接口")
@RestController
@RequestMapping("component/statistics")
public class ComponentStatisticsController {

    @Autowired
    private ComponentStatisticsService componentStatisticsService;

    @Autowired
    private BladeRedis bladeRedis;

    /**
     * 组件使用折线图
     */
    @GetMapping("/lineChart")
    @ApiOperation("组件使用折线图")
    public R lineChart(ComponentStatisticsParams params) {
        String parmJson = JSONUtil.toJsonStr(params);
        String cacheKey = CacheConstants.COMP_STAT_LINECHART_CACHE_KEY_PREFIX + DigestUtil.md5Hex(parmJson + AuthUtil.getTenantId());
        return R.data(bladeRedis.get(new CacheKey(cacheKey, Duration.ofSeconds(CacheConstants.COMP_STAT_CACHE_TIMEOUT_SEC)), () -> componentStatisticsService.lineChart(params)));
    }

    /**
     * 组件模板使用折线图
     */
    @GetMapping("/templateLineChart")
    @ApiOperation("组件模板使用折线图")
    public R templateLineChart(ComponentStatisticsParams params) {
        String parmJson = JSONUtil.toJsonStr(params);
        String cacheKey = CacheConstants.COMP_STAT_TEMPLATE_LINECHART_CACHE_KEY_PREFIX + DigestUtil.md5Hex(parmJson + AuthUtil.getTenantId());
        return R.data(bladeRedis.get(new CacheKey(cacheKey, Duration.ofSeconds(CacheConstants.COMP_STAT_CACHE_TIMEOUT_SEC)), () -> componentStatisticsService.templateLineChart(params)));
    }

    /**
     * 组件使用列表
     */
    @GetMapping("/componentList")
    @ApiOperation("组件使用列表")
    public R componentList(ComponentStatisticsParams params) {
        return R.data(componentStatisticsService.componentList(params));
    }

    @GetMapping("/componentListDownload")
    @ApiOperation("组件使用列表-导出")
    public void componentListDownload(ComponentStatisticsParams params, HttpServletResponse response){
        componentStatisticsService.componentListDownload(params, response);
    }

    /**
     * 组件top
     */
    @GetMapping("/componentTop")
    @ApiOperation("组件top")
    public R componentTop(
            @ApiParam("组件来源 0是系统组件 1是超管自定义组件 2用户自定义组件 3是组件模板-公共 4是组件模板-私有")
            @RequestParam(defaultValue = "0") Integer componentSource,
            @ApiParam("top多少")
            @RequestParam(defaultValue = "10") Integer top) {
        String cacheKey = String.format("%s%s:%s:%s", CacheConstants.COMP_STAT_TOPN_CACHE_KEY_PREFIX, componentSource, top, AuthUtil.getTenantId());
        return R.data(bladeRedis.get(new CacheKey(cacheKey, Duration.ofSeconds(CacheConstants.COMP_STAT_CACHE_TIMEOUT_SEC)), () -> componentStatisticsService.componentTop(componentSource,top)));
    }

    /**
     * 组件模板使用列表
     */
    @GetMapping("/componentTemplateList")
    @ApiOperation("组件模板使用列表")
    public R componentTemplateList(ComponentStatisticsParams params) {
        return R.data(componentStatisticsService.componentTemplateList(params));
    }

    @GetMapping("/componentTemplateListDownload")
    @ApiOperation("组件模板使用列表-导出")
    public void componentTemplateListDownload(ComponentStatisticsParams params, HttpServletResponse response){
        componentStatisticsService.componentTemplateListDownload(params, response);
    }

}
