/**
 *
 */
package org.springblade.visual.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import lombok.extern.slf4j.Slf4j;
import org.springblade.constant.CacheConstants;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.dto.*;
import org.springblade.visual.entity.VisualClassify;
import org.springblade.visual.entity.VisualClassifyFavourite;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springblade.visual.entity.VisualReportCategoryFavourite;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.entity.VisualSelfReportCategory;
import org.springblade.visual.enums.VisualDataTemplateCategory;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualDataTemplateCategoryMapper;
import org.springblade.visual.service.VisualClassifyFavouriteService;
import org.springblade.visual.service.VisualClassifyService;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualDataTemplateService;
import org.springblade.visual.service.VisualReportCategoryFavouriteService;
import org.springblade.visual.service.VisualSelfReportCategoryService;
import org.springblade.visual.service.VisualTemplateClassifyService;
import org.springblade.visual.service.VisualTemplateService;
import org.springblade.visual.vo.CopyVisualDataCubeDataVo;
import org.springblade.visual.vo.VisualClassifyVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author wookvn
 */
@Slf4j
@RestController
@RequestMapping("visual/data")
@Api(value = "大屏模版接口", tags = "大屏模版接口")
public class VisualDataController extends BladeController {

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private VisualClassifyService visualClassifyService;

    @Autowired
    private VisualTemplateService visualTemplateService;

    @Autowired
    private DataPermClient dataPermClient;

    @Autowired
    private VisualSelfReportCategoryService visualSelfReportCategoryService;

    @Autowired
    private VisualClassifyFavouriteService visualClassifyFavouriteService;

    @Autowired
    private VisualReportCategoryFavouriteService visualReportCategoryFavouriteService;

    @Autowired
    private VisualTemplateClassifyService visualTemplateClassifyService;

    @Autowired
    private VisualDataTemplateService visualDataTemplateService;

    @Autowired
    private VisualDataTemplateCategoryMapper visualDataTemplateCategoryMapper;

    @Autowired
    private BladeRedis bladeRedis;

    /**
     * 详情
     */
    @GetMapping("/detail")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "详情", notes = "传入visual")
    public R<VisualData> detail(@RequestParam Long id) {
//        String cacheKey = CacheConstants.VISUAL_DATA_CACHE_KEY_PREFIX + id;
//        // 从缓存查询大屏组件信息
//        VisualData visualData = bladeRedis.get(cacheKey);
//        if(visualData != null) {
//            return R.data(visualData);
//        }
        // 缓存不存在，从数据库查询
        VisualData detail = visualDataService.getById(id);
        if (detail == null) {
            return R.fail("数据不存在");
        }
        //查询大屏使用的所有组件ID
//        LambdaQueryWrapper<VisualComponent> queryWrapper = Wrappers.<VisualComponent>lambdaQuery()
//                .select(VisualComponent::getId)
//                .eq(VisualComponent::getVisualId, detail.getId())
//                .eq(VisualComponent::getIsMobile, 0);
//        List<Long> componentIdList = visualComponentService.listObjs(queryWrapper, a -> Long.valueOf(a.toString()));
//        detail.setComponentIds(componentIdList);
//        bladeRedis.setEx(cacheKey, detail, Duration.ofSeconds(CacheConstants.VISUAL_DATA_CACHE_TIMEOUT_SEC));
        return R.data(detail);
    }

    /**
     * @param visualData
     * @return
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "列表")
    public R<IPage<VisualData>> list(VisualData visualData, Query query) {
        Long userId = getUser().getUserId();
        LambdaQueryWrapper<VisualData> queryWrapper = Condition.getQueryWrapper(visualData).lambda().eq(VisualData::getVisualType, VisualTypeEnum.GENERAL).orderByDesc(VisualData::getCreateTime);
        queryWrapper.eq(VisualData::getVisualDataType, 0); //查询可视化大屏
        queryWrapper.select(VisualData::getVisualDesc,VisualData::getStatus,VisualData::getId, VisualData::getTitle, VisualData::getCreateTime, VisualData::getPublicStatus, VisualData::getReviewStatus, VisualData::getBackgroundUrl, VisualData::getCategory,VisualData::getIsMobile);
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
        if (checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.and(t -> {
                //创建人自己的大屏
                t.eq(VisualData::getCreateUser, userId)
                        //查询权限组的大屏
                        .or().inSql(VisualData::getId, "select a.visual_id from visual_data_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
                        // 查询公共大屏
                        .or().eq(VisualData::getPublicStatus, 1);
            });
        }
        return R.data(visualDataService.page(Condition.getPage(query), queryWrapper));
    }


    /**
     * @param visualData
     * @return
     */
    @GetMapping("/list/waterfall")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "列表")
    public R<List<VisualData>> waterfallList(VisualData visualData, Query query, String createTimeIndex) {
        Long userId = getUser().getUserId();
        visualData.setVisualDataType(0);
        if (visualData.getVisualType() == null) {
            visualData.setVisualType(VisualTypeEnum.GENERAL);
        }
        LambdaQueryWrapper<VisualData> queryWrapper = Condition.getQueryWrapper(visualData).lambda().orderByDesc(VisualData::getCreateTime);// .orderByDesc(VisualData::getId);
        queryWrapper.select(VisualData::getId, VisualData::getTitle, VisualData::getCreateTime, VisualData::getPublicStatus, VisualData::getReviewStatus, VisualData::getBackgroundUrl, VisualData::getCategory,VisualData::getIsMobile,VisualData::getStatus);
        queryWrapper.lt(VisualData::getCreateTime, createTimeIndex);
        queryWrapper.last("limit " + query.getSize());
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(userId);
        if (checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.and(t -> {
                //创建人自己的大屏
                t.eq(VisualData::getCreateUser, userId)
                        //查询权限组的大屏
                        .or().inSql(VisualData::getId, "select a.visual_id from visual_data_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
                        // 查询公共大屏
                        .or().eq(VisualData::getPublicStatus, 1);
            });
        }
        return R.data(visualDataService.list(queryWrapper));
    }



    /**
     * @param visualData
     * @return
     */
    @GetMapping("/myList")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "我的收藏列表")
    public R<IPage<VisualData>> mylist(VisualData visualData, Query query) {
        visualData.setCreateUser(getUser().getUserId());
        visualData.setVisualDataType(0);
        visualData.setVisualType(VisualTypeEnum.FAVOURITE);
        LambdaQueryWrapper<VisualData> queryWrapper = Condition.getQueryWrapper(visualData).lambda().orderByDesc(VisualData::getCreateTime);
        queryWrapper.select(VisualData::getId, VisualData::getTitle, VisualData::getBackgroundUrl, VisualData::getCategory,VisualData::getIsMobile);
        return R.data(visualDataService.page(Condition.getPage(query), queryWrapper));
    }

    /**
     * @param visualData
     * @return
     */
    @ApiLog("报表组件")
    @PostMapping("/update")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "更新大屏（仅更新大屏的数据，不传递参数不更新）")
    public R<String> update(@RequestBody VisualData visualData) {
        bladeRedis.del(CacheConstants.VISUAL_DATA_CACHE_KEY_PREFIX + visualData.getId());
        return R.status(visualDataService.updateById(visualData));
    }

    /**
     * @return
     * @throws IOException
     */
    @ApiLog("报表组件")
    @PostMapping("/submit")
    @ApiOperationSupport(order = 5)
    @ApiOperation(value = "新增或更新大屏", notes = "返回值：大屏ID")
    @ApiResponse(message = "大屏ID", code = 200)
    public R<Long> submit(@RequestBody String value) throws IOException {
        String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
        VisualDto visualDto = JSON.parseObject(decoderStr, VisualDto.class);
        VisualData visualData = visualDto.getVisualData();
        if (visualData.getVisualDataType() == null) {
            visualData.setVisualDataType(0);
        }
        Long visualId = visualDataService.doSaveOrUpdate(visualDto, getUser());
        bladeRedis.del(CacheConstants.VISUAL_DATA_CACHE_KEY_PREFIX + visualId, CacheConstants.COMP_LIST_BY_VISUALID_CACHE_KEY_PREFIX + visualId);
        return R.data(visualId);
    }

    /**
     * @param ids
     * @return
     */
    @ApiLog("报表组件")
    @PostMapping("/delete")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "删除大屏")
    public R<String> delete(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        List<Long> idList = Func.toLongList(ids);
        visualDataService.doRemove(idList);
        bladeRedis.del(idList.stream().map(id -> CacheConstants.VISUAL_DATA_CACHE_KEY_PREFIX + id).collect(Collectors.toList()));
        bladeRedis.del(idList.stream().map(id -> CacheConstants.COMP_LIST_BY_VISUALID_CACHE_KEY_PREFIX + id).collect(Collectors.toList()));
        return R.success("删除成功");
    }

    /**
     * 拷贝 可视化表
     */
    @ApiLog("报表组件")
    @PostMapping("/copy")
    @ApiOperationSupport(order = 7)
    @ApiOperation(value = "拷贝大屏模板")
    public R<Long> copy(
            @ApiParam(value = "原大屏id", required = true) @RequestParam Long id,
            @ApiParam(value = "复制大屏名称", required = false) @RequestParam(required = false) String visualName,
            @ApiParam(value = "新分类id", required = false) @RequestParam(required = false) Long newCategoryId,
            @ApiParam(value = "租户id") @RequestParam(required = false) String tenantId,
            @ApiParam(value = "大屏类型", required = false) @RequestParam(required = false) VisualTypeEnum visualType) {
        List<Long> visualIdList = visualDataService.doCopy(CollectionUtil.newHashSet(id), newCategoryId, null, null, visualType, getUser(), visualName, tenantId);
        return R.data(CollectionUtil.isNotEmpty(visualIdList) ? visualIdList.get(0) : null);
    }

    /**
     * 根据大屏id获取组件和数据集
     */
    @GetMapping("/component-cube")
    @ApiOperationSupport(order = 10)
    @ApiOperation(value = "根据大屏id获取组件和数据集")
    public R<List<VisualComponentCubeDataDto>> getComponentCubeDataByVisualId(Integer visualId) {
        return R.data(visualDataService.ComponentCubeData(visualId));
    }

    /**
     * 拷贝 可视化表
     */
    @ApiLog("报表组件")
    @PostMapping("/copy-cube")
    @ApiOperationSupport(order = 9)
    @ApiOperation(value = "拷贝大屏模板和组件数据集")
    public R<Integer> copyCube(@RequestBody CopyVisualDataCubeDataVo cubeData) {
        return visualDataService.doCopyCube(cubeData, getUser());
    }

    /**
     * 调整顺序
     *
     * @return
     */
    @ApiLog("报表组件")
    @PostMapping("/adjustOrder")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "调整顺序", notes = "传入大屏ID列表【切记：保证大屏顺序】")
    public R<String> uptOrder(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
        List<VisualData> entityList = new ArrayList<VisualData>();
        List<Long> visualIdList = Func.toLongList(ids);
        for (int i = 0; i < visualIdList.size(); i++) {
            VisualData data = new VisualData();
            data.setId(visualIdList.get(i));
            data.setOrderIndex(i);
            entityList.add(data);
        }
        return R.status(visualDataService.updateBatchById(entityList));
    }

    /**
     * 根据大屏id获取分类名称
     */
    @GetMapping("/get/classify")
    @ApiOperationSupport(order = 9)
    @ApiOperation(value = "根据大屏id查询大屏分类", notes = "传入大屏 id,type:(1:大屏类，2：模板)")
    public R<VisualClassifyVO> getClassifyByVisualId(@RequestParam(required = false) String visualId,
                                                     @RequestParam(required = false) String type) {
        VisualClassifyVO classify = new VisualClassifyVO();
        if (StrUtil.isEmpty(type) || (!type.equals("null") && type.equals("1"))) {
            // 先查询大屏信息
            VisualData visualData = visualDataService.getById(Long.parseLong(visualId));
            if (visualData == null || visualData.getId() < 0) {
                return null;
            }
            // 自助报表
            if (visualData.getVisualDataType() == 1) {
                // 我的收藏类型
                if (visualData.getVisualType().getValue() == VisualTypeEnum.FAVOURITE.getValue()) {
                    VisualReportCategoryFavourite favourite = Optional.ofNullable(visualReportCategoryFavouriteService.getById(visualData.getCategory())).orElse(new VisualReportCategoryFavourite());
                    classify.setId(favourite.getId());
                    classify.setName(favourite.getCategoryName());
                } else {
                    VisualSelfReportCategory category = Optional.ofNullable(visualSelfReportCategoryService.getById(visualData.getCategory())).orElse(new VisualSelfReportCategory());
                    classify.setId(category.getId());
                    classify.setName(category.getCategoryValue());
                }
            } else { // 普通大屏
                // 我的收藏类型
                if (visualData.getVisualType().getValue() == VisualTypeEnum.FAVOURITE.getValue()) {
                    VisualClassifyFavourite favourite = Optional.ofNullable(visualClassifyFavouriteService.getById(visualData.getCategory())).orElse(new VisualClassifyFavourite());
                    classify.setId(favourite.getId());
                    classify.setName(favourite.getClassifyName());
                } else {
                    VisualClassify visualClassify = Optional.ofNullable(visualClassifyService.getById(visualData.getCategory())).orElse(new VisualClassify());
                    classify.setId(visualClassify.getId());
                    classify.setName(visualClassify.getCategoryValue());
                }
            }
        } else {
            VisualDataTemplate visualDataTemplate = visualDataTemplateService.getBaseMapper().getByVisualDataId(Long.parseLong(visualId));
            if (visualDataTemplate != null) {
                VisualDataTemplateCategory visualDataTemplateCategory = visualDataTemplateCategoryMapper.selectById(visualDataTemplate.getCategory());
                if (visualDataTemplateCategory != null) {
                    classify.setId(visualDataTemplateCategory.getId());
                    classify.setName(visualDataTemplateCategory.getName());
                }
            }
        }

        return R.data(classify);
    }

    @PostMapping("/lock")
    @ApiOperation(value = "大屏锁")
    public VisualLockDto lLock(Integer visualId) {
        return visualDataService.visualLock(visualId);
    }

    @ApiOperation(value = "解锁")
    @PostMapping("/unlock")
    public boolean visualUnlock(@RequestBody VisualUnlockParams params) {
        return visualDataService.visualUnlock(params);
    }

    @ApiOperation(value = "发起审核")
    @PostMapping("/submitReview")
    public R submitReview(@RequestBody SubmitReviewParams params) {
        return R.status(visualDataService.submitReview(params));
    }

    @ApiOperation("获取审核记录")
    @GetMapping("/reviewLogs/{id}")
    public R<List<VisualReviewLog>> listReviewLogs(@PathVariable Long id) {
        return R.data(visualDataService.listReviewLogs(id));
    }

    @ApiOperation("添加修改模板")
    @PostMapping("/submitTemplate")
    public R<VisualDataTemplate> submitTemplate(@RequestBody String json) {
        return R.data(visualDataService.submitTemplate(json, getUser()));
    }

    @ApiOperation("通过id获取大屏访问量")
    @PostMapping("/visualViewCount")
    public R<List<VisualViewCountDto>> visualViewCount(@RequestBody VisualViewCountParams params){
        return R.data(visualDataService.visualViewCount(params));
    }

    @ApiOperation("开发者页面饼图统计")
    @PostMapping("/visualCount")
    public R<VisualCountDto> visualCount(){
        return R.data(visualDataService.visualCount());
    }

    @ApiOperation("开发者页面数量统计")
    @PostMapping("/createCount")
    public R<CreateCountDto> createCount(){
        return R.data(visualDataService.createCount());
    }

}
