/**
 *
 */
package org.springblade.visual.controller;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.record.DVALRecord;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.constant.TokenConstant;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.core.tool.utils.Base62Util;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.system.client.DataPermClient;
import org.springblade.system.client.IamClient;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.UserModel;
import org.springblade.visual.dto.VisualPowerInfoDto;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualPowerPointCategory;
import org.springblade.visual.entity.VisualPowerPointInfo;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

import javax.servlet.http.HttpServletRequest;

/**
 * 幻灯片 操作处理
 *
 * @author wookvn
 * @date 2023-02-04
 */
@Slf4j
@RestController
@RequestMapping("visual/powerPointInfo")
@Api(value = "幻灯片相关接口", tags = "幻灯片接口")
public class VisualPowerPointInfoController extends BladeController {

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private VisualPowerPointInfoService visualPowerPointInfoService;

    @Autowired
    private VisualPowerPointCategoryService visualPowerPointCategoryService;

    @Autowired
    private DataPermClient dataPermClient;
    @Autowired
    private IamClient iamClient;

    /**
     * 列表数据
     *
     * @param visualPowerPointInfo
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询幻灯片", httpMethod = "GET")
    public R<IPage<VisualPowerPointInfo>> pageList(VisualPowerPointInfo visualPowerPointInfo, Query query) {
        LambdaQueryWrapper<VisualPowerPointInfo> queryWrapper = Condition.getQueryWrapper(visualPowerPointInfo).lambda().eq(VisualPowerPointInfo::getVisualType, 0);
        Long userId = getUser().getUserId();
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(getUser().getUserId());
        if (checkSuperPermRet == null || !checkSuperPermRet) {
            queryWrapper.and(t -> t.eq(VisualPowerPointInfo::getCreateUserId, userId)
                    .or().inSql(VisualPowerPointInfo::getId, "select a.pp_id from visual_power_point_permission a inner join data_permission_user b on a.data_perm_id = b.data_perm_id where b.user_id = " + userId)
                    .or().eq(VisualPowerPointInfo::getPublicStatus, 1));
        }
        IPage<VisualPowerPointInfo> pageResult = visualPowerPointInfoService.page(Condition.getPage(query), queryWrapper.orderByDesc(VisualPowerPointInfo::getUpdateTime));
        return R.data(pageResult);
    }

    /**
     * @param powerPointInfo
     * @return
     */
    @GetMapping("/myList")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "我的收藏列表")
    public R<IPage<VisualPowerPointInfo>> mylist(VisualPowerPointInfo powerPointInfo, Query query) {
        powerPointInfo.setCreateUserId(getUser().getUserId());
        powerPointInfo.setVisualType(1);
        IPage<VisualPowerPointInfo> infoIPage = getVisualPowerPointInfoIPage(powerPointInfo, query);
        return R.data(infoIPage);
    }

    /**
     * @param powerPointInfo
     * @return
     */
    @GetMapping("/archiving-list")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "幻灯片归档列表")
    public R<IPage<VisualPowerPointInfo>> archiving(VisualPowerPointInfo powerPointInfo, Query query) {
        powerPointInfo.setCreateUserId(getUser().getUserId());
        powerPointInfo.setVisualType(2);
        IPage<VisualPowerPointInfo> infoIPage = getVisualPowerPointInfoIPage(powerPointInfo, query);
        return R.data(infoIPage);
    }

    /**
     * 单条数据查询接口
     *
     * @param id
     * @return
     */
    @GetMapping("/info")
    @ApiOperation(value = "获取幻灯片详情以及大屏详情", httpMethod = "GET")
    public R<Kv> info(@RequestParam Long id) {
        VisualPowerPointInfo powerPointInfo = visualPowerPointInfoService.getById(id);
        if (powerPointInfo == null) {
            return R.fail("数据不存在");
        }
        UserModel userModel = iamClient.getById(powerPointInfo.getCreateUserId(), Base62Util.decode(powerPointInfo.getTenantId()));
        if (userModel != null) {
            powerPointInfo.setCreateUserName(userModel.getNickName());
        }
//        if (CollectionUtil.isNotEmpty(visualList)) {
//            List<Long> visualIds = visualList.stream().map(VisualData::getId).collect(Collectors.toList());
//            List<VisualComponent> componentList = visualComponentService.lambdaQuery().select(VisualComponent::getId, VisualComponent::getVisualId)
//                    .in(VisualComponent::getVisualId, visualIds)
//                    .list();
//            if (CollectionUtil.isNotEmpty(componentList)) {
//                Map<Long, List<VisualComponent>> listMap = componentList.stream().collect(Collectors.groupingBy(VisualComponent::getVisualId));
//                visualList.forEach(item -> {
//                    List<VisualComponent> list = listMap.get(item.getId());
//                    if (CollectionUtil.isNotEmpty(list)) {
//                        List<Long> componentIds = list.stream().map(VisualComponent::getId).collect(Collectors.toList());
//                        item.setComponentIds(componentIds);
//                    }
//                });
//            }
//        }
        Kv detail = Kv.create();
        LambdaQueryWrapper<VisualData> queryWrapper = Wrappers.<VisualData>lambdaQuery()
                .eq(VisualData::getVisualType, VisualTypeEnum.POWER_POINT)
                .eq(VisualData::getPowerPointId, id).orderByAsc(VisualData::getOrderIndex);
        List<VisualData> visualList = visualDataService.list(queryWrapper);
        return R.data(detail.set("detail", powerPointInfo).set("visualList", visualList));
    }

    /**
     * 修改数据接口
     *
     * @param body
     * @return
     */
    @PostMapping("/submit")
    @ApiOperation(value = "增加或修改幻灯片基本信息", httpMethod = "POST")
    public R<String> doUpdate(@Validated @RequestBody String body) {
        VisualPowerInfoDto visualPowerInfoDto = JSONUtil.toBean(body, VisualPowerInfoDto.class);
        if (visualPowerInfoDto.getPpName().length() >= 100) {
            return R.fail("名字已超过100字限制");
        }
        if (StrUtil.isBlank(visualPowerInfoDto.getPpName())) {
            return R.fail("幻灯片名字不能为空");
        }
//		//名字判重
//		List<VisualPowerPointInfo> list = visualPowerPointInfoService.lambdaQuery()
//				.eq(VisualPowerPointInfo::getPpName, visualPowerInfoDto.getPpName())
//				.ne(visualPowerInfoDto.getId() != null, VisualPowerPointInfo::getId, visualPowerInfoDto.getId())
//				.list();
//		if (CollectionUtil.isNotEmpty(list)) {
//			return R.fail("名字重复");
//		}
        if (visualPowerInfoDto.getId() == null) {
            visualPowerInfoDto.setCreateTime(LocalDateTime.now());
            visualPowerInfoDto.setCreateUserId(getUser().getUserId());
        }
        visualPowerInfoDto.setUpdateBy(SecureUtil.getNickName());
        visualPowerInfoDto.setUpdateTime(LocalDateTime.now());
        return R.status(visualPowerPointInfoService.saveOrUpdatePower(visualPowerInfoDto, getUser()));
    }

    /**
     * 删除数据接口
     *
     * @param id
     * @return
     */
    @PostMapping("/delete")
    @ApiOperation(value = "根据ID删除幻灯片", httpMethod = "POST")
    public R<String> doDelete(@RequestParam Long id) {
        return R.status(visualPowerPointInfoService.doRemove(id));
    }

    /**
     * @return
     */
    @PostMapping("/addVisual")
    @ApiOperation(value = "添加幻灯片的大屏列表", httpMethod = "POST")
    public R<List<Long>> doUpdateVisual(
            HttpServletRequest request,
            @ApiParam(value = "幻灯片ID", required = true) @RequestParam Long ppId,
            @ApiParam(value = "大屏ID列表，多个逗号隔开", required = true) @RequestParam String visualIds) {
        return R.data(visualPowerPointInfoService.doAddVisual(ppId, visualIds, getUser()));
    }

    @PostMapping("/removeVisual")
    @ApiOperation(value = "批量删除幻灯片的大屏列表", httpMethod = "POST")
    public R<Boolean> doRemoveVisual(
            @ApiParam(value = "幻灯片ID", required = true) @RequestParam Long ppId,
            @ApiParam(value = "大屏ID列表，多个逗号隔开", required = true) @RequestParam String visualIds) {
        return R.data(visualPowerPointInfoService.doRemoveVisualList(ppId, visualIds, getUser()));
    }

    /**
     * 删除幻灯片中的某一个大屏
     *
     * @param ppId
     * @param visualId
     * @return
     */
    @PostMapping("/deleteVisual")
    @ApiOperation(value = "删除幻灯片的单张大屏", httpMethod = "POST")
    public R<String> doDeleteVisual(
            @ApiParam(value = "幻灯片ID", required = true) @RequestParam Long ppId,
            @ApiParam(value = "大屏ID", required = true) @RequestParam Long visualId) {
        return R.status(visualPowerPointInfoService.doRemoveVisual(ppId, visualId, getUser()));
    }

    private IPage<VisualPowerPointInfo> getVisualPowerPointInfoIPage(VisualPowerPointInfo powerPointInfo, Query query) {
        IPage<VisualPowerPointInfo> infoIPage = visualPowerPointInfoService.page(Condition.getPage(query), Condition.getQueryWrapper(powerPointInfo).lambda().orderByDesc(VisualPowerPointInfo::getUpdateTime));
        return infoIPage;
    }
}