/**
 * 
 */
package org.springblade.visual.mapper;

import java.util.List;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.springblade.visual.dto.TemplateTopDto;
import org.springblade.visual.dto.VisualTemplateLineChartDto;
import org.springblade.visual.entity.VisualData;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;

/**
 * @author wookvn
 *
 */
public interface VisualDataMapper extends BaseMapper<VisualData> {


	@Select({
			"<script>",
			"SELECT * FROM visual_data",
			"WHERE power_point_id IN ",
			"<foreach item='item' index='index' collection='ppIds' open='(' separator=',' close=')'>",
			"#{item}",
			"</foreach>",
			"ORDER BY order_index ASC, id ASC",
			"</script>"
	})
	@InterceptorIgnore(tenantLine = "true")
	List<VisualData> listByPpIds(@Param("ppIds") List<Long> ppIds);


	@Select({
			"<script>",
			"SELECT * FROM visual_data",
			"WHERE id IN ",
			"<foreach item='item' index='index' collection='ids' open='(' separator=',' close=')'>",
			"#{item}",
			"</foreach>",
			"ORDER BY order_index ASC, id ASC",
			"</script>"
	})
	@InterceptorIgnore(tenantLine = "true")
	List<VisualData> listByIds(@Param("ids") List<Long> ids);

	@Select({
		"select distinct c.category from visual_data c",
		"left join visual_data_permission a on c.id = a.visual_id",
		"left join data_permission_user b on a.data_perm_id = b.data_perm_id",
		"where (b.user_id = #{userId} or c.create_user = #{userId} or c.public_status = 1) and c.visual_data_type = #{visualDataType}"
	})
	public List<Long> classifyIdListPerm(@Param("userId") Long userId,@Param("visualDataType") Integer visualDataType);


//	@InterceptorIgnore(tenantLine = "true")
//	@Select("select * from visual_data ${ew.customSqlSegment}")
//	public VisualData getDetail(@Param(Constants.WRAPPER)QueryWrapper<?> queryWrapper);


	/**
	 * 查询使用模板的大屏列表
	 * @return
	 */
	@Select("<script>" +
            "SELECT d.*,t.name as templateName FROM visual_data d LEFT JOIN visual_data_template t ON d.template_id = t.id where d.template_id IS NOT NULL " +
            "<if test='templateName != null'> AND t.name like CONCAT('%', #{templateName}, '%') </if>" +
            "<if test='tenantId != null'> AND d.tenant_id = #{tenantId} </if>" +
            "<if test='startDate != null and endDate != null'> AND DATE(d.create_time) BETWEEN #{startDate} AND #{endDate} </if>" +
            " order by d.create_time desc" +
            "</script>")
	Page<VisualData> pageTemplateVisual(Page page, @Param("templateName") String templateName,
										@Param("tenantId") String tenantId,
										@Param("startDate") String startDate,
										@Param("endDate") String endDate);

	@Select("<script>" +
			"SELECT d.*,t.name as templateName FROM visual_data d LEFT JOIN visual_data_template t ON d.template_id = t.id where d.template_id IS NOT NULL " +
			"<if test='templateName != null'> AND t.name like CONCAT('%', #{templateName}, '%') </if>" +
			"<if test='tenantId != null'> AND d.tenant_id = #{tenantId} </if>" +
			"<if test='startDate != null and endDate != null'> AND DATE(d.create_time) BETWEEN #{startDate} AND #{endDate} </if>" +
			" order by d.create_time desc" +
			"</script>")
	List<VisualData> listTemplateVisual(@Param("templateName") String templateName,
										@Param("tenantId") String tenantId,
										@Param("startDate") String startDate,
										@Param("endDate") String endDate);

	@Select("<script>" +
			"SELECT DATE_FORMAT(d.create_time, '%Y-%m-%d') AS date, COUNT(*) AS count, COUNT(DISTINCT d.create_user) AS userCount FROM  visual_data d LEFT JOIN visual_data_template t ON d.template_id = t.id where d.template_id IS NOT NULL " +
			"<if test='templateName != null'> AND t.name like CONCAT('%', #{templateName}, '%') </if>" +
			"<if test='tenantId != null'> AND d.tenant_id = #{tenantId} </if>" +
			"<if test='startDate != null and endDate != null'> AND DATE(d.create_time) BETWEEN #{startDate} AND #{endDate} </if>" +
			"GROUP BY date order by date" +
			"</script>")
	List<VisualTemplateLineChartDto> lineChart(@Param("templateName") String templateName,
											   @Param("tenantId") String tenantId,
											   @Param("startDate") String startDate,
											   @Param("endDate") String endDate);

	@Select("SELECT template_id as id, COUNT(*) AS count, COUNT(DISTINCT create_user) AS userCount FROM visual_data WHERE template_id is not null  GROUP BY template_id ORDER BY count DESC LIMIT #{top};")
	List<TemplateTopDto> templateTop(@Param("top") Integer top);

	@InterceptorIgnore(tenantLine = "true")
	@Update("UPDATE visual_data SET status = #{status}, public_status = #{publicStatus} WHERE id = #{id} ")
	void updateStatus( @Param("status") int status, @Param("publicStatus") int publicStatus, @Param("id") Long id);

}
