package org.springblade.visual.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.annotations.Param;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.system.client.IamClient;
import org.springblade.system.client.TenantClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.visual.dto.ComponentLineChartDto;
import org.springblade.visual.dto.ComponentStatisticsParams;
import org.springblade.visual.dto.ComponentTopDto;
import org.springblade.visual.dto.VisualComponentStatisticsVo;
import org.springblade.visual.entity.*;
import org.springblade.visual.mapper.McpVisualComponentDefineMapper;
import org.springblade.visual.mapper.McpVisualComponentSysMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.sql.Array;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ComponentStatisticsService {

    @Autowired
    private VisualComponentService visualComponentService;
    @Autowired
    private VisualDataService visualDataService;
    @Autowired
    private IamClient tenantClient;
    @Autowired
    private McpVisualComponentSysMapper mcpVisualComponentSysMapper;
    @Autowired
    private McpVisualComponentDefineMapper mcpVisualComponentDefineMapper;
    @Autowired
    private VisualComponentTemplateService visualComponentTemplateService;

    /**
     * 组件使用折线图
     */
    public List<ComponentLineChartDto> lineChart(ComponentStatisticsParams params) {
        return visualComponentService.getBaseMapper()
                .lineChart(params.getComponentType(),
                        params.getTenantId(),
                        params.getComponentName(),
                        StrUtil.isNotBlank(params.getStartDate()) ? params.getStartDate() + " 00:00:00" : params.getStartDate(),
                        StrUtil.isNotBlank(params.getEndDate()) ? params.getEndDate() + " 23:59:59" : params.getEndDate());
    }

    /**
     * 组件模板使用折线图
     */
    public List<ComponentLineChartDto> templateLineChart(ComponentStatisticsParams params) {
        return visualComponentService.getBaseMapper()
                .templateLineChart(params.getCategoryId(),
                        params.getTenantId(),
                        params.getComponentName(),
                        StrUtil.isNotBlank(params.getStartDate()) ? params.getStartDate() + " 00:00:00" : params.getStartDate(),
                        StrUtil.isNotBlank(params.getEndDate()) ? params.getEndDate() + " 23:59:59" : params.getEndDate());
    }

    private LambdaQueryWrapper<VisualComponent> commonWrapper(ComponentStatisticsParams params) {
        LambdaQueryWrapper<VisualComponent> wrapper = Wrappers.<VisualComponent>lambdaQuery()
                .eq(params.getComponentType() != null, VisualComponent::getComponentSource, params.getComponentType())
                .ne(params.getComponentType() == null, VisualComponent::getComponentSource, 3)
                .isNotNull(params.getComponentType() == null, VisualComponent::getComponentSource)
                .eq(params.getTenantId() != null, TenantEntity::getTenantId, params.getTenantId())
                .eq(StringUtils.isNotBlank(params.getComponentName()), VisualComponent::getTitle, params.getComponentName())
                .between(params.getStartDate() != null && params.getEndDate() != null, VisualComponent::getCreateTime, params.getStartDate(), params.getEndDate() + " 23:59:59")
                .orderByDesc(VisualComponent::getCreateTime);
        return wrapper;
    }

    /**
     * 组件使用列表
     */
    public Page componentList(ComponentStatisticsParams params) {
        Page<VisualComponent> page = visualComponentService.page(new Page(params.getPage(), params.getSize()), commonWrapper(params));
        if (page.getRecords().size() != 0) {
            List<Long> visualIds = page.getRecords().stream().map(VisualComponent::getVisualId).collect(Collectors.toList());
            List<VisualData> visualData = visualDataService.listByIds(visualIds);
            Map<Long, String> visualMap = visualData.stream().collect(Collectors.toMap(VisualData::getId, VisualData::getTitle));
            List<String> tenantIds = page.getRecords().stream().map(TenantEntity::getTenantId).collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            for (VisualComponent visualComponent : page.getRecords()) {
                visualComponent.setContent(null);
                visualComponent.setVisualName(visualMap.get(visualComponent.getVisualId()));
                visualComponent.setTenantName(tenantMap.get(visualComponent.getTenantId()));
            }
        }
        return page;
    }


    public void componentListDownload(ComponentStatisticsParams params, HttpServletResponse response) {
        List<VisualComponent> list = visualComponentService.list(commonWrapper(params));
        if (list.size() != 0) {
            List<Long> visualIds = list.stream().map(VisualComponent::getVisualId).collect(Collectors.toList());
            List<VisualData> visualData = visualDataService.listByIds(visualIds);
            Map<Long, String> visualMap = visualData.stream().collect(Collectors.toMap(VisualData::getId, VisualData::getTitle));
            List<String> tenantIds = list.stream().map(TenantEntity::getTenantId).collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            List<VisualComponentStatisticsVo> visualComponentStatisticsVos = new ArrayList<>();
            for (VisualComponent visualComponent : list) {
                visualComponent.setContent(null);
                visualComponent.setVisualName(visualMap.get(visualComponent.getVisualId()));
                visualComponent.setTenantName(tenantMap.get(visualComponent.getTenantId()));
                VisualComponentStatisticsVo visualComponentStatisticsVo = new VisualComponentStatisticsVo();
                visualComponentStatisticsVos.add(visualComponentStatisticsVo);
                visualComponentStatisticsVo
                        .setComponentName(visualComponent.getTitle())
                        // 0是系统组件 1是超管自定义组件 2用户自定义组件 3是组件模板
                        .setComponentType("系统组件")
                        .setUser(visualComponent.getCreateBy())
                        .setTenant(visualComponent.getTenantName())
                        .setVisualName(visualComponent.getVisualName())
                        .setTime(DateUtil.format(visualComponent.getCreateTime(), DatePattern.NORM_DATETIME_PATTERN));
                if (visualComponent.getComponentSource() == 1) {
                    visualComponentStatisticsVo.setComponentType("系统自定义组件");
                } else if (visualComponent.getComponentSource() == 2) {
                    visualComponentStatisticsVo.setComponentType("自定义组件");
                }
            }

            String fileName = "/tmp/excel/" + "组件使用统计-" + DateUtil.format(new Date(), DatePattern.PURE_DATETIME_PATTERN) + ".xlsx";
            File directory = new File("/tmp/excel/");
            // 如果路径不存在则创建
            if (!directory.exists()) {
                directory.mkdirs(); // 创建多级目录
            }
            File file = new File(fileName);
            try {
                EasyExcel.write(fileName, VisualComponentStatisticsVo.class).sheet("组件使用统计").doWrite(visualComponentStatisticsVos);
                InputStream stream = new FileInputStream(file);
                // 设置响应头
                // 解决文件名中文乱码问题
                String encodedFileName = URLEncoder.encode(file.getName(), "UTF-8").replaceAll("\\+", "%20");
                response.setHeader("Content-Disposition", "attachment; filename=" + encodedFileName);
                response.setContentType("application/octet-stream");

                // 将流写入响应输出流
                byte[] buffer = new byte[1024];
                int bytesRead;
                OutputStream outputStream = response.getOutputStream();
                while ((bytesRead = stream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
                stream.close();
                outputStream.close();
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            } finally {
                try {
                    if(file.exists()){
                        file.delete();
                    }
                } catch (Exception e) {}
            }
        }
    }


    /**
     * 组件模板列表
     *
     * @param params
     * @return
     */
    public Page componentTemplateList(ComponentStatisticsParams params) {
        Page<VisualComponent> page = visualComponentService.getBaseMapper().componentTemplatePage(
                params.getCategoryId(),
                params.getTenantId(),
                params.getComponentName(),
                StrUtil.isNotBlank(params.getStartDate()) ? params.getStartDate() + " 00:00:00" : params.getStartDate(),
                StrUtil.isNotBlank(params.getEndDate()) ? params.getEndDate() + " 23:59:59" : params.getEndDate(),
                new Page(params.getPage(), params.getSize()));
        if (page.getRecords().size() != 0) {
            List<Long> visualIds = page.getRecords().stream().map(VisualComponent::getVisualId).collect(Collectors.toList());
            List<VisualData> visualData = visualDataService.listByIds(visualIds);
            Map<Long, String> visualMap = visualData.stream().collect(Collectors.toMap(VisualData::getId, VisualData::getTitle));
            List<String> tenantIds = page.getRecords().stream().map(TenantEntity::getTenantId).collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            for (VisualComponent visualComponent : page.getRecords()) {
                visualComponent.setContent(null);
                visualComponent.setVisualName(visualMap.get(visualComponent.getVisualId()));
                visualComponent.setTenantName(tenantMap.get(visualComponent.getTenantId()));
            }
            if (CollectionUtil.isNotEmpty(page.getRecords())) {
                List<Long> templateIds = page.getRecords().stream().map(VisualComponent::getComponentSourceId).distinct().collect(Collectors.toList());
                List<VisualComponentTemplate> visualComponentTemplates = visualComponentTemplateService.listByIds(templateIds);
                Map<Long, String> nameMap = visualComponentTemplates.stream().collect(Collectors.toMap(VisualComponentTemplate::getId, VisualComponentTemplate::getCreateUserName));
                if (nameMap != null && nameMap.keySet().size() != 0) {
                    for (VisualComponent visualComponent : page.getRecords()) {
                        visualComponent.setComponentTemplateCreateUser(nameMap.get(visualComponent.getComponentSourceId()));
                    }
                }
            }
        }
        return page;
    }


    public void componentTemplateListDownload(ComponentStatisticsParams params, HttpServletResponse response) {
        List<VisualComponent> list = visualComponentService.getBaseMapper().componentTemplateList(
                params.getCategoryId(),
                params.getTenantId(),
                params.getComponentName(),
                StrUtil.isNotBlank(params.getStartDate()) ? params.getStartDate() + " 00:00:00" : params.getStartDate(),
                StrUtil.isNotBlank(params.getEndDate()) ? params.getEndDate() + " 23:59:59" : params.getEndDate());
        if (list.size() != 0) {
            List<Long> visualIds = list.stream().map(VisualComponent::getVisualId).collect(Collectors.toList());
            List<VisualData> visualData = visualDataService.listByIds(visualIds);
            Map<Long, String> visualMap = visualData.stream().collect(Collectors.toMap(VisualData::getId, VisualData::getTitle));
            List<String> tenantIds = list.stream().map(TenantEntity::getTenantId).collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            for (VisualComponent visualComponent : list) {
                visualComponent.setContent(null);
                visualComponent.setVisualName(visualMap.get(visualComponent.getVisualId()));
                visualComponent.setTenantName(tenantMap.get(visualComponent.getTenantId()));
            }
            if (CollectionUtil.isNotEmpty(list)) {
                List<Long> templateIds = list.stream().map(VisualComponent::getComponentSourceId).distinct().collect(Collectors.toList());
                List<VisualComponentTemplate> visualComponentTemplates = visualComponentTemplateService.listByIds(templateIds);
                Map<Long, String> nameMap = visualComponentTemplates.stream().collect(Collectors.toMap(VisualComponentTemplate::getId, VisualComponentTemplate::getCreateUserName));
                if (nameMap != null && nameMap.keySet().size() != 0) {
                    for (VisualComponent visualComponent : list) {
                        visualComponent.setComponentTemplateCreateUser(nameMap.get(visualComponent.getComponentSourceId()));
                    }
                }
            }
            List<VisualComponentStatisticsVo> visualComponentStatisticsVos = new ArrayList<>();
            for (VisualComponent visualComponent : list) {
                visualComponent.setContent(null);
                visualComponent.setVisualName(visualMap.get(visualComponent.getVisualId()));
                visualComponent.setTenantName(tenantMap.get(visualComponent.getTenantId()));
                VisualComponentStatisticsVo visualComponentStatisticsVo = new VisualComponentStatisticsVo();
                visualComponentStatisticsVos.add(visualComponentStatisticsVo);
                visualComponentStatisticsVo
                        .setComponentName(visualComponent.getName())
                        .setComponentType("组件模板")
                        .setUser(visualComponent.getCreateBy())
                        .setTenant(visualComponent.getTenantName())
                        .setVisualName(visualComponent.getVisualName())
//                        .setDate(DateUtil.format(visualComponent.getCreateTime(), DatePattern.NORM_DATE_FORMAT))
                        .setTime(DateUtil.format(visualComponent.getCreateTime(), DatePattern.NORM_DATETIME_PATTERN));
            }

            String fileName = "/tmp/excel/" + "组件模板使用统计-" + DateUtil.format(new Date(), DatePattern.PURE_DATETIME_PATTERN) + ".xlsx";
            File directory = new File("/tmp/excel/");
            // 如果路径不存在则创建
            if (!directory.exists()) {
                directory.mkdirs(); // 创建多级目录
            }
            File file = new File(fileName);
            try {
                EasyExcel.write(fileName, VisualComponentStatisticsVo.class).sheet("组件使用统计").doWrite(visualComponentStatisticsVos);
                InputStream stream = new FileInputStream(file);
                // 设置响应头
                // 解决文件名中文乱码问题
                String encodedFileName = URLEncoder.encode(file.getName(), "UTF-8").replaceAll("\\+", "%20");
                response.setHeader("Content-Disposition", "attachment; filename=" + encodedFileName);
                response.setContentType("application/octet-stream");

                // 将流写入响应输出流
                byte[] buffer = new byte[1024];
                int bytesRead;
                OutputStream outputStream = response.getOutputStream();
                while ((bytesRead = stream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
                stream.close();
                outputStream.close();
                file.delete();
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
    }

    /**
     * 组件top10
     */
    public List<ComponentTopDto> componentTop(Integer componentSource, Integer top) {
        List<ComponentTopDto> componentTopDtos = visualComponentService.getBaseMapper().componentTop(componentSource, top);
        Map<Long, String> map = new HashMap<>();
        if (CollectionUtil.isNotEmpty(componentTopDtos)) {
            List<Long> ids = componentTopDtos.stream().map(ComponentTopDto::getId).collect(Collectors.toList());
            if (componentSource == 0) {
                List<McpVisualComponentSys> mcpVisualComponentSys = mcpVisualComponentSysMapper.selectBatchIds(ids);
                map = mcpVisualComponentSys.stream().collect(Collectors.toMap(McpVisualComponentSys::getId, McpVisualComponentSys::getComponentName));
            } else if (componentSource == 1) {
                List<McpVisualComponentDefine> mcpVisualComponentDefines = mcpVisualComponentDefineMapper.selectBatchIds(ids);
                map = mcpVisualComponentDefines.stream().collect(Collectors.toMap(McpVisualComponentDefine::getId, McpVisualComponentDefine::getComponentName));
            } else if (componentSource == 3) {
                List<VisualComponentTemplate> visualComponentTemplates = visualComponentTemplateService.listByIds(ids);
                map = visualComponentTemplates.stream().collect(Collectors.toMap(template -> template.getId(), VisualComponentTemplate::getName));
            }
            for (ComponentTopDto componentTopDto : componentTopDtos) {
                componentTopDto.setName(map.get(componentTopDto.getId()));
            }
        }
        return componentTopDtos;
    }


}
