package org.springblade.visual.service;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.core.mp.base.TenantEntity;
import org.springblade.system.client.IamClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.visual.dto.*;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class VisualTemplateStatisticsService {

    @Autowired
    private VisualDataService visualDataService;
    @Autowired
    private VisualDataTemplateService visualDataTemplateService;
    @Autowired
    private IamClient tenantClient;

    /**
     * 模板使用折线图
     */
    public List<VisualTemplateLineChartDto> lineChart(VisualTemplateStatisticsParams params) {
        return visualDataService.getBaseMapper()
                .lineChart(
                        params.getTemplateName(),
                        params.getTenantId(),
                        params.getStartDate(),
                        params.getEndDate());
    }


    /**
     * 模板使用列表
     */
    public Page templateList(VisualTemplateStatisticsParams params) {
        Page<VisualData> page = visualDataService.getBaseMapper().pageTemplateVisual(new Page(params.getPage(), params.getSize()),
                params.getTemplateName(),
                params.getTenantId(),
                params.getStartDate(),
                params.getEndDate());
        if (page.getRecords().size() != 0) {
            List<String> tenantIds = page.getRecords().stream().map(TenantEntity::getTenantId).distinct().collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            List<Long> templateIds = page.getRecords().stream().map(VisualData::getTemplateId).collect(Collectors.toList());
            List<VisualDataTemplate> visualDataTemplates = visualDataTemplateService.listByIds(templateIds);
            Map<Long, VisualDataTemplate> templateMap = visualDataTemplates.stream().collect(Collectors.toMap(VisualDataTemplate::getId, Function.identity()));
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            for (VisualData visualData : page.getRecords()) {
                visualData.setContent(null);
                visualData.setTenantName(tenantMap.get(visualData.getTenantId()));
                if (templateMap.get(visualData.getTemplateId().longValue()) != null) {
                    visualData.setTemplateCreateUserName(templateMap.get(visualData.getTemplateId().longValue()).getCreateUserName());
                }
            }
        }
        return page;
    }

    public void templateListDownload(VisualTemplateStatisticsParams params, HttpServletResponse response) {
        List<VisualData> list = visualDataService.getBaseMapper().listTemplateVisual(
                params.getTemplateName(),
                params.getTenantId(),
                params.getStartDate(),
                params.getEndDate());
        if (list.size() != 0) {
            List<String> tenantIds = list.stream().map(TenantEntity::getTenantId).distinct().collect(Collectors.toList());
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            List<Long> templateIds = list.stream().map(VisualData::getTemplateId).collect(Collectors.toList());
            List<VisualDataTemplate> visualDataTemplates = visualDataTemplateService.listByIds(templateIds);
            Map<Long, VisualDataTemplate> templateMap = visualDataTemplates.stream().collect(Collectors.toMap(VisualDataTemplate::getId, Function.identity()));
            Map<String, String> tenantMap = tenantByTenantIds.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            List<VisualTemplateStatisticsExportDto> dtos = new ArrayList<>();
            for (VisualData visualData : list) {
                visualData.setContent(null);
                visualData.setTenantName(tenantMap.get(visualData.getTenantId()));
                if (templateMap.get(visualData.getTemplateId().longValue()) != null) {
                    visualData.setTemplateCreateUserName(templateMap.get(visualData.getTemplateId().longValue()).getCreateUserName());
                }
                VisualTemplateStatisticsExportDto dto = new VisualTemplateStatisticsExportDto();
                dtos.add(dto);
                dto
                        .setTemplateName(visualData.getTemplateName())
                        .setTemplateUrl("/newbi/template/preview/" + visualData.getId())
                        .setUser(visualData.getCreateUserName())
                        .setTenant(visualData.getTenantName())
                        .setCreateUserName(visualData.getTemplateCreateUserName())
                        .setTime(visualData.getCreateTime().format(DatePattern.NORM_DATETIME_FORMATTER));
            }
            String fileName = "/tmp/excel/" + "模版使用统计-" + DateUtil.format(new Date(), DatePattern.PURE_DATETIME_PATTERN) + ".xlsx";
            File directory = new File("/tmp/excel/");
            // 如果路径不存在则创建
            if (!directory.exists()) {
                directory.mkdirs(); // 创建多级目录
            }
            File file = new File(fileName);
            try {
                EasyExcel.write(fileName, VisualTemplateStatisticsExportDto.class).sheet("模板使用统计").doWrite(dtos);
                InputStream stream = new FileInputStream(file);
                // 解决文件名中文乱码问题
                String encodedFileName = URLEncoder.encode(file.getName(), "UTF-8").replaceAll("\\+", "%20");
                // 设置响应头
                response.setHeader("Content-Disposition", "attachment; filename=" + encodedFileName);
                response.setContentType("application/octet-stream");

                // 将流写入响应输出流
                byte[] buffer = new byte[1024];
                int bytesRead;
                OutputStream outputStream = response.getOutputStream();
                while ((bytesRead = stream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
                stream.close();
                outputStream.close();
                file.delete();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 模板top10
     */
    public List<TemplateTopDto> templateTop(Integer top) {
        List<TemplateTopDto> componentTopDtos = visualDataService.getBaseMapper().templateTop(top);
        if (componentTopDtos.size() != 0) {
            List<Long> ids = componentTopDtos.stream().map(TemplateTopDto::getId).collect(Collectors.toList());
            List<VisualDataTemplate> visualDataTemplates = visualDataTemplateService.listByIds(ids);
            if (visualDataTemplates.size() != 0) {
                Map<Long, String> map = visualDataTemplates.stream().collect(Collectors.toMap(VisualDataTemplate::getId, VisualDataTemplate::getName));
                for (TemplateTopDto componentTopDto : componentTopDtos) {
                    componentTopDto.setName(map.get(componentTopDto.getId()));
                }
            }
        }
        return componentTopDtos;
    }


}
