package org.springblade.visual.utils;

import com.alibaba.fastjson2.JSON;
import org.springblade.config.properties.AutoReviewProperties;
import org.springblade.system.client.dto.IamGetUserByIdsParams;
import org.springblade.visual.dto.IamResponse;
import org.springblade.visual.dto.YxConclusionCheckDto;
import org.springblade.visual.dto.YxConclusionCheckParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Collections;
import java.util.List;

@Component
public class YxApiUtils {

    private RestTemplate restTemplate = new RestTemplate();

    @Autowired
    private AutoReviewProperties autoReviewProperties;

    /**
     * 审核接口
     * http://swagger-im.9n1m.net/doc.html#/conclusion-servant/%E5%86%85%E5%AE%B9%E5%AE%A1%E6%A0%B8Controller/batchCheckV1UsingPOST
     * 	建议动作，0：通过，1：嫌疑，2：不通过
     */
    public YxConclusionCheckDto conclusionCheck(YxConclusionCheckParams params){
        ResponseEntity<IamResponse> response = restTemplate.postForEntity(autoReviewProperties.getVisualReviewUrl(), params, IamResponse.class);
        if (!checkResponse(response)) {
            throw new RuntimeException("YX-模板自动审核 接口请求失败--" + response);
        }
        if (response.getBody().getData() != null) {
            String jsonString = JSON.toJSONString(response.getBody().getData());
            List<YxConclusionCheckDto> yxConclusionCheckDtos = JSON.parseArray(jsonString, YxConclusionCheckDto.class);
            return yxConclusionCheckDtos.get(0);
        }
        return null;
    }

    private boolean checkResponse(ResponseEntity<IamResponse> response) {
        if (!response.getStatusCode().is2xxSuccessful() || !response.getBody().isSuccess()) {
            return false;
        }
        return true;
    }

}
